package org.greenstone.gatherer.util;

import java.io.*;
import org.greenstone.gatherer.Gatherer;

public class DecodeHTMLReader
    extends PushbackReader {

    public DecodeHTMLReader(Reader source) {
	super(source, 4);
    }

    /** Read a single character. */
    public int read() {
	return decode();
    }
 
    /** Read characters into a portion of an array. */
    public int read(char[] cbuf, int off, int len) {
	int count = 0;
	for(int i = off; i < len && ready(); i++) {
	    cbuf[i] = (char)decode();
	    count++;
	}
	return count;
    }

    public boolean ready() {
	try {
	    return super.ready();
	}
	catch (Exception error) {
	}
	return false;
    }

    /** Retrieve the next character off the stream. Unfortunately I have to do this a character at a time (which is slow). I also have to keep in mind that if a suspect encoded character turns out not to be I have to replace the extra characters. */
    private int decode() {
	int character;
	try {
	    character = super.read();
	}
	catch (Exception error) {
	    character = ' ';
	}
	try {
	    if(character == '&') {
		int amp = super.read();
		switch(amp) {
		case 'a':
		case 'A':
		    int ampap = super.read();
		    int ampapo = super.read();
		    int ampapos = super.read();
		    int ampapossemi = super.read();
		    if((ampap == 'p' || ampap == 'P') && (ampapo == 'o' || ampapo == 'O') && (ampapos == 's' || ampapos == 'S') && ampapossemi == ';') {
			// Read an &apos; so return an apostrophy
			return '\'';
		    }
		    // Not a apos. Return the characters removed in the correct order.
		    super.unread(ampapossemi);
		    super.unread(ampapos);
		    super.unread(ampapo);
		    super.unread(ampap);
		    break;
		case 'g':
		case 'G':
		    int ampgt = super.read();
		    int ampgtsemi = super.read();
		    if((ampgt == 't' || ampgt == 'T') && ampgtsemi == ';') {
			return '>';
		    }
		    super.unread(ampgtsemi);
		    super.unread(ampgt);
		    break;
		case 'l':
		case 'L':
		    int amplt = super.read();
		    int ampltsemi = super.read();
		    if((amplt == 't' || amplt == 'T') && ampltsemi == ';') {
			return '<';
		    }
		    super.unread(ampltsemi);
		    super.unread(amplt);
		    break;
		case 'q':
		case 'Q':
		    int ampqu = super.read();
		    int ampquo = super.read();
		    int ampquot = super.read();
		    int ampquotsemi = super.read();
		    if((ampqu == 'u' || ampqu == 'U') && (ampquo == 'o' || ampquo == 'O') && (ampquot == 't' || ampquot == 'T') && ampquotsemi == ';') {
			return '\"';
		    }
		    super.unread(ampquotsemi);
		    super.unread(ampquot);
		    super.unread(ampquo);
		    super.unread(ampqu);
		    break;
		case '#':
		    int amphash = super.read();
		    int amphash3 = super.read();
		    int amphash39 = super.read();
		    int amphash39semi = super.read();
		    if(amphash == '#' && amphash3 == '3' && amphash39 == '9' && amphash39semi == ';') {
			return '\'';
		    }
		    super.unread(amphash39semi);
		    super.unread(amphash39);
		    super.unread(amphash3);
		    super.unread(amphash);
		    break;
		}
		// Not a suspect. Return the character removed.
		super.unread(amp);
	    }
	}
	catch (Exception error) {
	    Gatherer.printStackTrace(error);
	}
	// Nothing special. Simply return the character extracted.
	return character;
    }
}
