/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

 




/* GPL_HEADER */
package org.greenstone.gatherer.util;
/**************************************************************************************
 * Title:        Gatherer
 * Description:  The Gatherer: a tool for gathering and enriching a digital collection.
 * Company:      The University of Waikato
 * Written:      16/05/02
 * Revised:      04/10/02 - Commented
 **************************************************************************************/
import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
/** This listener is used to ensure that in a group of JList or similar components, only one list actually has elements selected. 
 * @author John Thompson
 * @version 2.3
 */
public class ExclusiveListSelectionListener 
    implements ListSelectionListener {
    /** A flag set to show that we are already dealing with a list selection change, and to ignore further ones until the flag becomes unset again. */
    private boolean ignore = false;
    /** An array of JList components, containing each JList thats registered with this listener. */
    private JList lists[] = null;
    /** Method to register a new JList with this listener.
     * @param list The new <strong>JList</strong> to register.
	  */
    public void add(JList list) {
	list.addListSelectionListener(this);
	if(lists == null) {
	    lists = new JList[1];
	    lists[0] = list;
	}
	else {
	    JList temp[] = new JList[lists.length + 1];
	    System.arraycopy(lists, 0, temp, 0, lists.length);
	    temp[lists.length] = list;
	    lists = temp;
	}
    }
    /** Any implementation of ListSelectionListener must include this method so we can be informed when the selection changes. In this case we want to clear the selections of all other registered JLists other than the caller.
	  * @param event A <strong>ListSelectionEvent</strong> containing informaiton about the event.
	  */
    public void valueChanged(ListSelectionEvent event) {
	if(!ignore) {
	    ignore = true;
	    JList source = (JList)event.getSource();
	    for(int i = 0; i < lists.length; i++) {
		if(lists[i] != source) {
		    lists[i].clearSelection();
		}
	    }
	    ignore = false;
	}
    }
}
