package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element;

import java.util.HashMap;
import java.io.File;

public class PageAction extends Action {

    public static final String HOME_PAGE = "home";
    public static final String ABOUT_PAGE = "about";

    public Element process (Element message) {
	
	Element request = (Element) GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);
	// the page name is the subaction
	String page_name = request.getAttribute(GSXML.SUBACTION_ATT);
	if (page_name.equals("")) { // if no page specified, assume home page
	    page_name = HOME_PAGE;
	}
	Element result = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element response;
	if (page_name.equals(HOME_PAGE)) {
	    response = homePage(request);
	} else if (page_name.equals(ABOUT_PAGE)) {
	    response = aboutPage(request);
	} else { // unknown page
	
	    System.err.println("PageAction Error: unknown page specified!");
	    response =  unknownPage(request);
	}

	result.appendChild(doc_.importNode(response, true));
	///ystem.out.println("page action result: "+converter_.getPrettyString(result));
	return result;
    }


    protected Element homePage(Element request) {
	
	String lang = request.getAttribute(GSXML.LANG_ATT);
	
	// first, get the message router info
	Element coll_list_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element coll_list_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE, "", lang);
	coll_list_message.appendChild(coll_list_request);
	Element coll_list_response = (Element)mr_.process(coll_list_message);
	if (coll_list_response==null) {
	    System.err.println("PageAction Error: couldn't query the message router!");
	    return null;
	}
	
	// second, get the metadata for each collection - we only want specific
	// elements but for now, we'll just get it all
	NodeList colls = coll_list_response.getElementsByTagName(GSXML.COLLECTION_ELEM);
	// we will send all the requests in a single message
	Element metadata_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	for (int i=0; i<colls.getLength(); i++) {
	    Element c = (Element)colls.item(i);
	    String name = c.getAttribute(GSXML.NAME_ATT);	    
	    
	    Element metadata_request = GSXML.createBasicRequest(doc_,  GSXML.REQUEST_TYPE_DESCRIBE, name, lang);
	    metadata_message.appendChild(metadata_request);
	}

	Element metadata_response = (Element)mr_.process(metadata_message);

	NodeList coll_responses = metadata_response.getElementsByTagName(GSXML.RESPONSE_ELEM);
	// check that have same number of responses as collections
	if (colls.getLength() != coll_responses.getLength()) {
	    System.err.println("PageAction Error: didn't get a response for each collection - somethings gone wrong!");
	    // for now, dont use the metadata
	} else {
	    for (int i=0; i<colls.getLength(); i++) {
		Element c1 = (Element)colls.item(i);
		Element c2 = (Element)coll_responses.item(i);
		if (c1.getAttribute(GSXML.NAME_ATT).equals(c2.getAttribute(GSXML.FROM_ATT))) {
		    //add the collection data into the original response
		    GSXML.mergeElements(c1, (Element)GSXML.getChildByTagName(c2, GSXML.COLLECTION_ELEM));
		} else {
		    System.err.println("PageAction Error: response does not correspond to request!");
		} 
		
	    }
	}
	
	//now the full response message is in coll_list_response
	// return the response element
	Element response = (Element) GSXML.getChildByTagName(coll_list_response, GSXML.RESPONSE_ELEM);
	return response;
	
    } // homePage

    protected Element aboutPage(Element request) {

	String lang = request.getAttribute(GSXML.LANG_ATT);
	// extract the params from the cgi-request, 
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_paramList, false);
	
	String coll_name = (String)params.get(GSCGI.COLLECTION_ARG);
	if (coll_name == null || coll_name.equals("")) {
	    System.err.println("PageAction Error: about page requested with no collection or cluster specified!");
	    // return an empty response
	    return doc_.createElement(GSXML.RESPONSE_ELEM);
	}
	
	// get the collection or cluster description
	Element coll_about_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	
	Element coll_about_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE,  coll_name, lang);
	coll_about_message.appendChild(coll_about_request);
	
	Element coll_about_response = (Element)mr_.process(coll_about_message);

	if (coll_about_response == null) {
	    return null;
	}
	
	// second, get the info for each service - we only want display items 
	// but for now, we'll just get it all
	NodeList services = coll_about_response.getElementsByTagName(GSXML.SERVICE_ELEM);
	// we will send all the requests in a single message
	Element info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	for (int i=0; i<services.getLength(); i++) {
	    Element c = (Element)services.item(i);
	    String name = c.getAttribute(GSXML.NAME_ATT);
	    String address = GSPath.appendLink(coll_name, name);
	    Element info_request = GSXML.createBasicRequest(doc_,  GSXML.REQUEST_TYPE_DESCRIBE, address, lang);
	    info_message.appendChild(info_request);
	}

	Element info_response = (Element)mr_.process(info_message);

	NodeList service_responses = info_response.getElementsByTagName(GSXML.RESPONSE_ELEM);
	// check that have same number of responses as collections
	if (services.getLength() != service_responses.getLength()) {
	    System.err.println("PageAction Error: didn't get a response for each service - somethings gone wrong!");
	    // for now, dont use the metadata
	} else {
	    for (int i=0; i<services.getLength(); i++) {
		Element c1 = (Element)services.item(i);
		Element c2 = (Element)GSXML.getChildByTagName((Element)service_responses.item(i), GSXML.SERVICE_ELEM);
		if (c1.getAttribute(GSXML.NAME_ATT).equals(c2.getAttribute(GSXML.NAME_ATT))) {
		    //add the service data into the original response
		    GSXML.mergeElements(c1, c2);
		} else {
		    System.err.println("PageAction Error: response does not correspond to request!");
		} 
		
	    }
	}
	
	//now the full response message is in coll_list_response
	// return the response element
	Element response = (Element) GSXML.getChildByTagName(coll_about_response, GSXML.RESPONSE_ELEM);
	return response;
    }


    /** if we dont know the page type, use this method */
    protected Element unknownPage(Element request) {
	
	String lang = request.getAttribute(GSXML.LANG_ATT);
	String page_name = request.getAttribute(GSXML.SUBACTION_ATT);

	// extract the params from the cgi-request, 
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_paramList, false);
	
	String coll_name = (String)params.get(GSCGI.COLLECTION_ARG);
	if (coll_name == null || coll_name.equals("")) {
	    // just return an empty response
	    return doc_.createElement(GSXML.RESPONSE_ELEM);
	}
	
	// else get the coll description - actually this is the same as for the about page - should we merge these two methods??
	
	// if there is a service specified should we get the service description instead??
	// get the collection or cluster description
	Element coll_about_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	
	Element coll_about_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE,  coll_name, lang);
	coll_about_message.appendChild(coll_about_request);
	
	Element coll_about_response = (Element)mr_.process(coll_about_message);

	// add the response to the page get teh response element and return it
	Element response = (Element) GSXML.getChildByTagName(coll_about_response, GSXML.RESPONSE_ELEM);
	return response;
	
    }
}
