package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Text; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.util.HashSet;
import java.util.Vector;
import java.util.Map;
import java.util.Iterator;
import java.io.File;

/** action class for queries */
public class QueryAction extends Action {
    
    /** process - processes a request.
     */ 
    public Element process (Element message) {

	// get the request - assume there is only one
	Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);

	// create the return message
	Element result = doc_.createElement(GSXML.MESSAGE_ELEM);
	// for now we only have one type of query - subaction not used
	Element response = basicQuery(request);
	result.appendChild(doc_.importNode(response, true));
	return result;
    }
    
    /** a generic query handler 
     * this gets the service description, does the query (just passes all teh 
     * params to the service, then gets the titles for any results 
     */
    protected Element basicQuery(Element request) {
	
	// the result
	Element page_response = doc_.createElement(GSXML.RESPONSE_ELEM);
	
	// extract the params from the cgi-request, and check that we have a coll specified
	Element cgi_param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_param_list, false);
	
	String request_type = (String)params.get(GSCGI.REQUEST_TYPE_ARG);
	String service_name = (String)params.get(GSCGI.SERVICE_ARG);
	String collection = (String)params.get(GSCGI.COLLECTION_ARG);
	
	if (collection == null || collection.equals("")) {
	    System.err.println("QueryAction Error: no collection was specified!");
	    return page_response; // an empty response 
	}
	
	String lang = request.getAttribute(GSXML.LANG_ATT);
	String to = GSPath.appendLink(collection, service_name);
	
	// part of the response is the service description 
	// for now get this again from the service. 
	// this will probably need to be cached somehow later on. 
	Element mr_info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_info_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE,  to, lang);
	mr_info_message.appendChild(mr_info_request);

	// also get the format stuff now if there is some
	Element format_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_FORMAT, to, lang);
	mr_info_message.appendChild(format_request);

	// process the messages
	Element mr_info_response = (Element) mr_.process(mr_info_message);
	
	// the two responses
	NodeList responses = mr_info_response.getElementsByTagName(GSXML.RESPONSE_ELEM);
	Element service_response = (Element)responses.item(0);
	Element format_response = (Element)responses.item(1);
	
	Element service_description = (Element)doc_.importNode(GSXML.getChildByTagName(service_response, GSXML.SERVICE_ELEM), true);
	page_response.appendChild(service_description);
	
	if (request_type.equals("d")) {// just a display request
	    return page_response;
	}
	
	HashSet metadata_names = new HashSet();
	// add in the default elements
	metadata_names.add("Title");
	// add in the format info to the stylesheet if there is any
	Element format_elem = (Element)GSXML.getChildByTagName(format_response, GSXML.FORMAT_ELEM);
	if (format_elem != null) {
	    ///ystem.out.println("QueryAction: found a format element, adding it to the page response");
	    // set teh format type
	    format_elem.setAttribute(GSXML.TYPE_ATT, "search");
	    // for now just add to the response
	    page_response.appendChild(doc_.importNode(format_elem, true));
	    extractMetadataNames(format_elem, metadata_names);
	}
	

	// do the query
	Element mr_query_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_query_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_PROCESS, to, lang);
	mr_query_message.appendChild(mr_query_request);
	
	// paramList
	HashMap service_params = (HashMap)params.get("s1");
	if (service_params == null) { // no query
	    return page_response;
	}
	Element query_param_list = GSXML.createParameterList(doc_, service_params);//(Element)doc_.importNode(cgi_param_list, true);
	System.out.println("service params are "+converter_.getString(query_param_list));
	mr_query_request.appendChild(query_param_list);

	// do the query
       	Element mr_query_response = (Element)mr_.process(mr_query_message);

	String path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.METADATA_ELEM+GSXML.LIST_MODIFIER);
	Element query_result_metadata_list = (Element) GSXML.getNodeByPath(mr_query_response, path);
	if (query_result_metadata_list == null) {
	    System.err.println("QueryAction: Warning: No query result metadata.\n");
	} else { // add it into the page response
	    page_response.appendChild(doc_.importNode(query_result_metadata_list, true)); 
	}
	
	path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.TERM_ELEM+GSXML.LIST_MODIFIER);
	Element query_term_info_list = (Element) GSXML.getNodeByPath(mr_query_response, path);
	if (query_term_info_list == null) {
	    System.err.println("QueryAction: Warning: No query term information.\n");
	} else { // add it into the page response
	    page_response.appendChild(doc_.importNode(query_term_info_list, true));
	}

	// check that there are some documents - for now check the list, but later should use a numdocs metadata elem
	path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	
	Element document_list = (Element)GSXML.getNodeByPath(mr_query_response,
							     path); 
	// documentList not present if no docs found
	if (document_list == null) {
	    return page_response;
	}
	
	// do the metadata request
	Element mr_metadata_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_metadata_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_metadata_message.appendChild(mr_metadata_request);
	
	mr_metadata_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_PROCESS);
	mr_metadata_request.setAttribute(GSXML.LANG_ATT, lang);
	to = GSPath.appendLink(collection, "DocumentMetadataRetrieve"); // Hard-wired?
	mr_metadata_request.setAttribute(GSXML.TO_ATT, to);

	// just get all for now - the receptionist should perhaps pass in some
	// metadata that it wants, and QueryAction should look through the format stuff to see if there is any other?
	
	Element dm_param_list  = createMetadataParamList(metadata_names);
	
	mr_metadata_request.appendChild(dm_param_list);
	
	
	// add in the doc node list too
	mr_metadata_request.appendChild(doc_.importNode(document_list, true));
	Element mr_metadata_response = (Element) mr_.process(mr_metadata_message);	
	
	path = GSPath.appendLink(GSXML.RESPONSE_ELEM, GSXML.DOC_NODE_ELEM+GSXML.LIST_MODIFIER);
	Element query_result_document_list = (Element) GSXML.getNodeByPath(mr_metadata_response, path);

	if (query_result_document_list != null) {
	    page_response.appendChild(doc_.importNode(query_result_document_list, true));
	}

	// System.out.println("Query page:\n" + converter_.getPrettyString(page_response));
	return page_response;
    }
}
