/*
 *    GS2Construct.java 
 *    Copyright (C) 2002 New Zealand Digital Library, http://www.nzdl.org
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.greenstone.gsdl3.service;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.build.*;

import org.w3c.dom.Document; 
import org.w3c.dom.Node; 
import org.w3c.dom.Text; 
import org.w3c.dom.Element; 
import org.w3c.dom.NodeList; 

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.List;
import java.util.ArrayList;
import java.io.File;
import java.util.Locale;

/**
 * A Services class for building collections
 * provides a wrapper around the old perl scripts
 *
 * @author <a href="mailto:kjdon@cs.waikato.ac.nz">Katherine Don</a>
 * @version $Revision: 4903 $
 */
public class GS2Construct 
    extends ServiceRack { 
    
    // services offered
    private static final String NEW_SERVICE = "NewCollection";
    private static final String ADD_DOC_SERVICE = "AddDocument";
    private static final String IMPORT_SERVICE = "ImportCollection";
    private static final String BUILD_SERVICE = "BuildCollection";
    private static final String ACTIVATE_SERVICE = "ActivateCollection";
    private static final String DELETE_SERVICE = "DeleteCollection";
    private static final String RELOAD_SERVICE = "ReloadCollection";

    // params used
    private static final String COL_PARAM = "collection";
    private static final String NEW_COL_TITLE_PARAM = "collTitle";
    private static final String NEW_COL_ABOUT_PARAM = "collAbout";
    private static final String CREATOR_PARAM = "creator";
    private static final String NEW_FILE_PARAM = "newfile";
    private static final String PROCESS_ID_PARAM = GSCGI.PROCESS_ID_ARG;
    private static final String BUILDTYPE_PARAM = "buildType";
    private static final String BUILDTYPE_MG = "mg";
    private static final String BUILDTYPE_MGPP = "mgpp";
    
    // the list of the collections - store between some method calls
    private String[] collection_list_ = null;

    // set of listeners for any construction commands
    protected Map listeners_ = null;
    
    public GS2Construct() {
	listeners_ = Collections.synchronizedMap(new HashMap());
	
    }

    /** returns a specific service description */
    protected Element getServiceDescription(String service, String lang) {

	Element description = doc_.createElement(GSXML.SERVICE_ELEM);
	description.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_PROCESS);
	description.setAttribute(GSXML.NAME_ATT, service);
	description.appendChild(GSXML.createDisplayTextElement(doc_, GSXML.DISPLAY_TEXT_NAME,  getTextString(service+".name", lang)));
	description.appendChild(GSXML.createDisplayTextElement(doc_,  GSXML.DISPLAY_TEXT_SUBMIT, getTextString(service+".submit", lang)));

	Element param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	description.appendChild(param_list);
	
	if (service.equals(NEW_SERVICE)) {
	    
	    Element param = GSXML.createParameterDescription(doc_, NEW_COL_TITLE_PARAM, getTextString("param."+NEW_COL_TITLE_PARAM, lang), GSXML.PARAM_TYPE_STRING, null, null, null);
	    param_list.appendChild(param);
	    param = GSXML.createParameterDescription(doc_, CREATOR_PARAM, getTextString("param."+CREATOR_PARAM, lang), GSXML.PARAM_TYPE_STRING, null, null, null);
	    param_list.appendChild(param);
	    param = GSXML.createParameterDescription(doc_, NEW_COL_ABOUT_PARAM, getTextString("param."+NEW_COL_ABOUT_PARAM, lang), GSXML.PARAM_TYPE_TEXT, null, null, null);
	    param_list.appendChild(param);
	    String[] types = {BUILDTYPE_MGPP, BUILDTYPE_MG};
	    String[] type_texts = {getTextString("param."+BUILDTYPE_PARAM+"."+BUILDTYPE_MGPP, lang), getTextString("param."+BUILDTYPE_PARAM+"."+BUILDTYPE_MG, lang)};
	    
	    param = GSXML.createParameterDescription(doc_, BUILDTYPE_PARAM, getTextString("param."+BUILDTYPE_PARAM, lang), GSXML.PARAM_TYPE_ENUM_SINGLE, BUILDTYPE_MGPP, types, type_texts);
	    param_list.appendChild(param);
	    return description;
	}
	
	
	if (service.equals(ACTIVATE_SERVICE) || service.equals(IMPORT_SERVICE) || service.equals(BUILD_SERVICE) || service.equals(RELOAD_SERVICE) || service.equals(DELETE_SERVICE)) {
	    
	    collection_list_ = getCollectionList();
	    Element param = GSXML.createParameterDescription(doc_, COL_PARAM, getTextString("param."+COL_PARAM, lang), GSXML.PARAM_TYPE_ENUM_SINGLE, null, collection_list_, collection_list_);	
	    param_list.appendChild(param);
	    
	    return description;
	}
	// invalid service name
	return null;
	
    }

    // each service must have a method "process<New service name>"

    protected Element processNewCollection(Element request) {
	return runCommand(request, GS2PerlConstructor.NEW);	
    }
    
    /** TODO:implement this */
    protected Element processAddDocument(Element request) {
	// decode the file name, add it to the import directory
	String name = GSPath.getFirstLink(request.getAttribute(GSXML.TO_ATT));
	Element response = doc_.createElement(GSXML.RESPONSE_ELEM);
	response.setAttribute(GSXML.FROM_ATT, name);
	Element status = doc_.createElement(GSXML.STATUS_ELEM);
	response.appendChild(status);
	//String lang = request.getAttribute(GSXML.LANG_ATT);
	//String request_type = request.getAttribute(GSXML.TYPE_ATT);
	Text t = doc_.createTextNode("AddDocument: not implemented yet");
	status.appendChild(t);
	status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.ERROR));
	return response;
    }
    protected Element processImportCollection(Element request) {
	return runCommand(request, GS2PerlConstructor.IMPORT);
    }

    protected Element processBuildCollection(Element request) {
	return runCommand(request, GS2PerlConstructor.BUILD);
    }
    protected Element processActivateCollection(Element request) {
	Element response = runCommand(request, GS2PerlConstructor.ACTIVATE);
	// this activates the collection on disk. but now we need to tell
	// the MR about it. but we have to wait until the process is finished.
	Element status = (Element)GSXML.getChildByTagName(response, GSXML.STATUS_ELEM);
	Element param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(param_list, false);
	String coll_name = (String) params.get(COL_PARAM);
	String lang = request.getAttribute(GSXML.LANG_ATT);
	// check for finished
	int status_code = Integer.parseInt(status.getAttribute(GSXML.STATUS_ERROR_CODE_ATT));
	if (GSStatus.isCompleted(status_code) && 
	    GSStatus.isError(status_code)) {
	    // we shouldn't carry out the next bit, just return the response
	    return response;
	}
	String id = status.getAttribute(GSXML.STATUS_PROCESS_ID_ATT);
	GS2PerlListener listener = (GS2PerlListener)listeners_.get(id);
	if (listener==null) {
	    System.err.println("somethings gone wrong, couldn't find the listener");
	    status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.ERROR));
	    return response;
	}
	while (!GSStatus.isCompleted(status_code)) {
	    // wait for the process, and keep checking the status code
	    // there is probably a better way to do this.
	    try {
		Thread.currentThread().sleep(100);
	    } catch (Exception e) { // ignore
	    }
	    status_code = listener.getStatus();
	}
	
	// add the rest of the messages to the status node
	Text t = doc_.createTextNode("\n"+listener.getUpdate());
	status.appendChild(t);
	status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(listener.getStatus()));
	if (GSStatus.isError(status_code)) {
	    return response; // without doing the next bit
	}

	t = doc_.createTextNode("\n");
	status.appendChild(t);
	// once have got here, we assume
	// the first bit proceeded successfully, now reload the collection
	configureRequest("reload", coll_name, status, lang); // this will append more messages to the status, and overwrite the error code att
	return response;
	
    }
    protected Element processDeleteCollection(Element request) {

	// the response to send back
	String name = GSPath.getFirstLink(request.getAttribute(GSXML.TO_ATT));
	Element response = doc_.createElement(GSXML.RESPONSE_ELEM);
	response.setAttribute(GSXML.FROM_ATT, name);
	Element status = doc_.createElement(GSXML.STATUS_ELEM);
	response.appendChild(status);
	Text t = null; // the text node for the error/success message
	String lang = request.getAttribute(GSXML.LANG_ATT);
	String request_type = request.getAttribute(GSXML.TYPE_ATT);
	
	Element param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(param_list, false);
	
	boolean get_status_only = false;
	if (request_type.equals(GSXML.REQUEST_TYPE_STATUS)) {
	    get_status_only = true;
	}
	if (get_status_only) {
	    // at the moment, delete is synchronous. but it may take ages so should do the command in another thread maybe? in which case we will want to ask for status
	    System.err.println("had a status request for delete - this shouldn't happen!!");
	    //t = doc_.createTextNode("");
	    //status.appendChild(t);
	    status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.ERROR));
	    return response;
	}
	String coll_name = (String)params.get(COL_PARAM);	
	String[] args={coll_name};
	File coll_dir = new File(GSFile.collectionBaseDir(site_home_, coll_name));
	// check that the coll is there in the first place
	if (!coll_dir.exists()) {
	    t = doc_.createTextNode(getTextString("delete.exists_error", args, lang));
	    status.appendChild(t);
	    status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.ERROR));
	    return response;
	}
		
	// try to delete the directory
	if (!GSFile.deleteFile(coll_dir)) {
	    t = doc_.createTextNode(getTextString("delete.delete_error", args, lang));
	    status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.ERROR));
	    status.appendChild(t);
	    return response;
	}
	configureRequest("delete", coll_name, status, lang);
	return response;
    }

    protected Element processReloadCollection(Element request) {

	// the response to send back
	String name = GSPath.getFirstLink(request.getAttribute(GSXML.TO_ATT));
	Element response = doc_.createElement(GSXML.RESPONSE_ELEM);
	response.setAttribute(GSXML.FROM_ATT, name);
	Element status = doc_.createElement(GSXML.STATUS_ELEM);
	response.appendChild(status);
	Text t = null; // the text node for the error/success message
	
	String lang = request.getAttribute(GSXML.LANG_ATT);
	String request_type = request.getAttribute(GSXML.TYPE_ATT);
	
	Element param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(param_list, false);
	
	boolean get_status_only = false;
	if (request_type.equals(GSXML.REQUEST_TYPE_STATUS)) {
	    get_status_only = true;
	}
	if (get_status_only) {
	    // reload is synchronous - this makes no sense
	    System.err.println("had a status request for reload - this shouldn't happen!!");
	    //t = doc_.createTextNode("");
	    //status.appendChild(t);
	    status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.ERROR));
	    return response;
	}
	
	String coll_name = (String)params.get(COL_PARAM);
	configureRequest("reload", coll_name, status, lang);
	return response;	
	
    }

    /** send a configure request to the message router 
     * action name should be "delete" or "reload"
     * response will be put into the status element
     */
    protected void configureRequest(String action_name, String coll_name, Element status, String lang) {
	
	// send the request to the MR
	Element message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element request = doc_.createElement(GSXML.REQUEST_ELEM);
	request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_CONFIGURE);
	request.setAttribute(GSXML.TO_ATT, "");
	message.appendChild(request);
	Element command = doc_.createElement(GSXML.CONFIGURE_ELEM);
	command.setAttribute(GSXML.TYPE_ATT, GSXML.COLLECTION_ELEM);
	command.setAttribute(GSXML.NAME_ATT, coll_name);
	if (action_name.equals("delete")) {
	    command.setAttribute(GSXML.ACTION_ATT, GSXML.CONFIG_ACTION_DEACTIVATE);
	} else if (action_name.equals("reload")) {
	    command.setAttribute(GSXML.ACTION_ATT, GSXML.CONFIG_ACTION_ACTIVATE);
	} else {
	    System.err.println("invalid action name passed to configureRequest:"+action_name);
	    return;
	}
	request.appendChild(command);
	Element response = router_.process(message); // at the moment, get no info in response so ignore it
	Text t;
	String[] args={coll_name};
	    
	if (response == null) {
	    t = doc_.createTextNode(getTextString(action_name+".configure_error", args , lang));
	    status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.ERROR));
	    status.appendChild(t);
	    return;
	}
	    
	// if we got here, we have succeeded!
	t = doc_.createTextNode(getTextString(action_name+".success", args, lang));
	status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.SUCCESS));
	status.appendChild(t);
    }


    /** configure the service module
     * for now, all services have type=build - need to think about this
     */
    public boolean configure(Element info, Element extra_info) {
	System.out.println("configuring GS2Construct");

	Element e = null;
	// hard code in the services for now

	// set up short_service_info_ - for now just has name and type

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_PROCESS);
	e.setAttribute(GSXML.NAME_ATT, NEW_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_PROCESS);
	e.setAttribute(GSXML.NAME_ATT, IMPORT_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_PROCESS);
	e.setAttribute(GSXML.NAME_ATT, BUILD_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_PROCESS);
	e.setAttribute(GSXML.NAME_ATT, ACTIVATE_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_PROCESS);
	e.setAttribute(GSXML.NAME_ATT, DELETE_SERVICE);
	short_service_info_.appendChild(e);

	e = doc_.createElement(GSXML.SERVICE_ELEM);
	e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_PROCESS);
	e.setAttribute(GSXML.NAME_ATT, RELOAD_SERVICE);
	short_service_info_.appendChild(e);

	//e = doc_.createElement(GSXML.SERVICE_ELEM);
	//e.setAttribute(GSXML.TYPE_ATT, GSXML.SERVICE_TYPE_PROCESS);
	//e.setAttribute(GSXML.NAME_ATT, ADD_DOC_SERVICE);
	//short_service_info_.appendChild(e);


	return true;
    }

    /** returns a response element */
    protected Element runCommand(Element request, int type ) {
	
	// the response to send back
	String name = GSPath.getFirstLink(request.getAttribute(GSXML.TO_ATT));
	Element response = doc_.createElement(GSXML.RESPONSE_ELEM);
	response.setAttribute(GSXML.FROM_ATT, name);
	Element status = doc_.createElement(GSXML.STATUS_ELEM);
	response.appendChild(status);

	String lang = request.getAttribute(GSXML.LANG_ATT);
	String request_type = request.getAttribute(GSXML.TYPE_ATT);

	Element param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(param_list, false);

	boolean get_status_only = false;
	if (request_type.equals(GSXML.REQUEST_TYPE_STATUS)) {
	    get_status_only = true;
	}
	
	// just check for status messages if that's all that's required
	if (get_status_only) {
	    String id = (String)params.get(PROCESS_ID_PARAM);
	    status.setAttribute(GSXML.STATUS_PROCESS_ID_ATT, id);
	    GS2PerlListener listener = (GS2PerlListener)listeners_.get(id);
	    if (listener==null) {
		Text t = doc_.createTextNode(getTextString("general.process_id_error", lang));
		status.appendChild(t);
		status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.ERROR));
	    } else {
		Text t = doc_.createTextNode(listener.getUpdate());
		status.appendChild(t);
		status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(listener.getStatus()));
		// check that we actually should be removing the listener here
		if (listener.isFinished()) { // remove this listener - its job is done
		    listeners_.remove(id); // not working
		}
	    }
	    return response;

	}

	// do teh actual command
	String coll_name=null;
	if (type==GS2PerlConstructor.NEW) {
	    String coll_title = (String)params.get(NEW_COL_TITLE_PARAM);
	    coll_name = createNewCollName(coll_title);
	} else {
	    coll_name = (String)params.get(COL_PARAM);
	}
	System.err.println("Coll name = "+coll_name);
	// makes a paramList of the relevant params
	Element other_params = extractOtherParams(params, type);

	//create the constructor to do the work

	GS2PerlConstructor constructor = new GS2PerlConstructor("perl_build");
	if (!constructor.configure()) {
	    Text t = doc_.createTextNode(getTextString("general.configure_constructor_error", lang));
	    status.appendChild(t);
	    status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.ERROR));
	    return response;
	}
		
	constructor.setSiteHome(site_home_);
	constructor.setCollectionName(coll_name);
	constructor.setActionType(type);
	constructor.setProcessParams(other_params);
	// the listener
	GS2PerlListener listener = new GS2PerlListener();
	constructor.addListener(listener);
	constructor.start();

	String id = newID();
	listeners_.put(id, listener);

	status.setAttribute(GSXML.STATUS_PROCESS_ID_ATT, id);
	status.setAttribute(GSXML.STATUS_ERROR_CODE_ATT, Integer.toString(GSStatus.ACCEPTED));
	Text t = doc_.createTextNode(getTextString("general.process_start", lang));
	status.appendChild(t);
	return response;

    }


    //************************
    // some helper functions
    //************************

    /** parse the collect directory and return a list of collection names */
    protected String[] getCollectionList() {
	
	File collectDir = new File(GSFile.collectDir(site_home_));
	if (!collectDir.exists()) {
	    System.err.println("couldn't find collect dir: "+collectDir.toString());
	    return null;
	}
	System.out.println("GS2Construct: reading thru directory "+collectDir.getPath()+" to find collections.");
	File[] contents = collectDir.listFiles();
	int num_colls=0;
	for (int i=0; i<contents.length;i++) {
	    if(contents[i].isDirectory() && !contents[i].getName().startsWith("CVS")) {
		num_colls ++;
	    }
	}
	
	String[] names= new String[num_colls];
	
	for (int i=0, j=0; i<contents.length;i++) {
	    if(contents[i].isDirectory()) {
		String colName = contents[i].getName();
		if (!colName.startsWith("CVS")) {
		    names[j] = colName;
		    j++;
		}
		
	    } 
	}
	
	return names;
	
    }
   
    /** ids used for process id */
    private int current_id = 0;
    private String newID() {
	current_id++;
	return Integer.toString(current_id);
    }

    /** creates a new short name from the collection title */
    protected String createNewCollName(String coll_title) {

	String base_name = null;
	// take the first 6 letters
	if (coll_title.length()<6) {
	    base_name = coll_title;
	} else {
	    base_name = coll_title.substring(0,6);
	}
	File coll_dir = new File(GSFile.collectionBaseDir(site_home_, base_name));
	if (!coll_dir.exists()) { // this name is ok - not used yet
	    return base_name;
	}
	
	// now we have to make a new name until we get a good one
	// try name1, name2 name3 etc
	int i=0;
	while(coll_dir.exists()) {
	    i++;
	    coll_dir = new File(GSFile.collectionBaseDir(site_home_, base_name+Integer.toString(i)));
	}
	return base_name+Integer.toString(i);
	
    }

    /** takes the params from the request (in the HashMap) and extracts any
     * that need to be passed to the constructor and puts them into a 
     * paramList element */
    protected Element extractOtherParams(HashMap params, int type) {

	Element param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	if (type == GS2PerlConstructor.NEW) {
	    Element param = doc_.createElement(GSXML.PARAM_ELEM);
	    param.setAttribute(GSXML.NAME_ATT, "creator");
	    param.setAttribute(GSXML.VALUE_ATT, (String)params.get(CREATOR_PARAM));
	    
	    param_list.appendChild(param);
	    param = doc_.createElement(GSXML.PARAM_ELEM);
	    param.setAttribute(GSXML.NAME_ATT, "about");
	    param.setAttribute(GSXML.VALUE_ATT, (String)params.get(NEW_COL_ABOUT_PARAM));
	    param_list.appendChild(param);
	    param = doc_.createElement(GSXML.PARAM_ELEM);
	    param.setAttribute(GSXML.NAME_ATT, "title");
	    param.setAttribute(GSXML.VALUE_ATT, (String)params.get(NEW_COL_TITLE_PARAM));
	    param_list.appendChild(param);
	    param = doc_.createElement(GSXML.PARAM_ELEM);
	    param.setAttribute(GSXML.NAME_ATT, "buildtype");
	    param.setAttribute(GSXML.VALUE_ATT, (String)params.get(BUILDTYPE_PARAM));
	    param_list.appendChild(param);
	    return param_list;
	}

	// other ones dont have params yet
	return null;
    }
	
}


