<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:java="http://xml.apache.org/xslt/java"
  xmlns:util="xalan://org.greenstone.gsdl3.util.XSLTUtil"
  extension-element-prefixes="java util"
  exclude-result-prefixes="java util">
  
  <!-- style includes global params interface_name, library_name -->
  <xsl:include href="style.xsl"/>
  
  <xsl:output method="html"/>  

  <xsl:template name="pageHead">
    <head>
      <title>x
	<xsl:value-of select="/page/pageResponse/*/displayItem[@name='name']"/>
      </title>
    </head>
  </xsl:template>

  <xsl:template match="pageResponse">
    <xsl:variable name="collName" select="/page/pageRequest/paramList/param[@name='c']/@value"/>
    <xsl:apply-templates select="collection|serviceCluster"><xsl:with-param name="collName" select="$collName"/></xsl:apply-templates>
  </xsl:template>
  
  <xsl:template match="collection|serviceCluster">
    <xsl:param name="collName"/>
    <xsl:if test='name()="collection"'>
      <xsl:call-template name="collectionPageBanner">
	<xsl:with-param name="collName" select="$collName"/>
      </xsl:call-template>
    </xsl:if>
    <xsl:if test='name()="serviceCluster"'>
      <xsl:call-template name="clusterPageBanner">
	<xsl:with-param name="clusterName" select="$collName"/>
      </xsl:call-template>
    </xsl:if>
    <p/><h2><xsl:value-of select="displayItem[@name='name']"/></h2>
    <p/><xsl:value-of select="displayItem[@name='description']"/>
    <p/><xsl:apply-templates select="metadataList">
      <xsl:with-param name="collName" select="$collName"/>
    </xsl:apply-templates>
    <p/><xsl:apply-templates select="serviceList">
      <xsl:with-param name="collName" select="$collName"/>
    </xsl:apply-templates>
  </xsl:template>
  
  
  <xsl:template match="metadataList">
    <xsl:param name="collName"/>
    <h2><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'about.metadata')"/></h2>
    <p/>
    <table width='537'>
      <xsl:for-each select="metadata">
	<tr valign='top'><td><b><xsl:value-of select="@name"/></b></td><td><xsl:value-of select="."/></td></tr>
      </xsl:for-each>
    </table>
  </xsl:template>
  
  <xsl:template match="serviceList">
    <xsl:param name="collName"/>
    <h2><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'about.services')"/></h2>
    <xsl:if test="service">
      <table>
	<xsl:for-each select="service">
	  <xsl:variable name="action"><xsl:choose>
	      <xsl:when test="@type='query'">q</xsl:when>
	      <xsl:when test="@type='browse'">b</xsl:when>
	      <xsl:when test="@type='process'">pr</xsl:when>
	      <xsl:when test="@type='applet'">a</xsl:when>
	      <xsl:otherwise>DO_NOT_DISPLAY</xsl:otherwise>
	    </xsl:choose></xsl:variable>
	  <xsl:if test="$action !='DO_NOT_DISPLAY'">
	    <tr><td><a href="{$library_name}?a={$action}&amp;rt=d&amp;s={@name}&amp;c={$collName}"><xsl:value-of select="displayItem[@name='name']"/></a></td></tr>
	  </xsl:if>
	</xsl:for-each>
      </table>
    </xsl:if>
    <xsl:call-template name="greenBar"/>
  </xsl:template>


</xsl:stylesheet>  

