/*
 *    NzdlDocumentInfo.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//the package we're in
package org.nzdl.gsdl.service;

/** gsdl corba stuff */
import org.nzdl.gsdl.corba.gsdlInterface.*;
import org.nzdl.gsdl.util.*;
import java.util.*;

/**
 * An object that stores information on a collection. Information is obtained from
 * {@link NzdlService NzdlService.getDocumentInfo(name)}
 * Easily interrogated by a user.
 *
 * @author Stuart Yeates  (say1@cs.waikato.ac.nz)
 * @author Aziz Mahoui    (amahoui@yahoo.com)
 * @author Gordon Paynter (paynter@cs.waikato.ac.nz)
 * @author Dave Nichols   (daven@cs.waikato.ac.nz)
 * @author Brett Sheeran (bsheeran@xtra.co.nz) (comments)
 * @version $Revision: 2663 $
 */
public class NzdlDocumentInfo extends Object {

  private Vector keys = new Vector();
  private Vector values = new Vector();

  /**
   * Create an instance of NzdlDocumentInfo.
   */
  public NzdlDocumentInfo(NzdlService nzdl, 
			  String  collectionName,
			  String docID ) {
    String[] fields= { 
      "Audio",
      "Author",
      "Creator",
      "Date",
      "DateText",
      "DocIcon",
      "Encoding",
      "Filename",
      "FiltText",
      "From",
      "GENERATOR",
      "Generator",
      "Headers",
      "Howto",
      "Identifier",
      "Image",
      "Issue",
      "Language",
      "Link",
      "Magazine",
      "Organization",
      "Page" ,
      "Series",
      "Source",
      "Subject",
      "Template",
      "Title", 
      "To",
      "Type", 
      "URL",
      "first200",
      "first100",
      "gsdlassocfile",
      "gsdldoctype",
      "gsdlnum", 
      "gsdlsourcefilename",
      "href",
      "srcicon",
      "srclink"
    };

    for(int i=0;i<fields.length;i++) {
     
      Set set = nzdl.getMetaData( collectionName, docID, fields[i] );
      Iterator iterator = set.iterator();
      while(iterator.hasNext()) {
	String value = (String) iterator.next();
	if (value != null && value != "" && value.length() > 0) {
	  keys.add(fields[i]);
	  values.add(value);
	}
      }
    }
  }

/**
 * Returns all metadata keys for each metadata value in the document.
 * Each item returned corresponds to a value in the 'values' Vector.
 *  The keys may include such Strings as title, author, creator etc.
 * Where there is more than one instance of a metadata value (for
 * example, if there are two authors) then two author keys will be
 * produced each corresponding to an author name in the values Vector.
 * @return an enumerated set of key strings
 */
  public Enumeration getKeys() {return keys.elements();};


 /**
 * Returns all metadata values in the document.
 * Each item returned corresponds to a key in the 'keys' Vector.
 *  The keys may include such Strings as title, author, creator etc.
 * @return an enumerated set of value strings
 */
  public Enumeration getValues() {return values.elements();};
}








